@@ -1,5 +1,5 @@
 /*
- * Copyright 1989, 1990, 1991, 1992, 1993, John F. Haugh II
+ * Copyright 1989 - 1994, John F. Haugh II
  * All rights reserved.
  *
  * Permission is granted to copy and create derivative works for any
@@ -9,7 +9,8 @@
  * distribution media.
  *
  * This software is provided on an AS-IS basis and the author makes
- * no warrantee of any kind.
+ * no warrantee of any kind.  The "Artistic License" located in the
+ * file LICENSE contains complete licensing information.
  */
 
 #include <sys/types.h>
@@ -20,7 +21,7 @@
 #include <time.h>
 
 #ifndef	lint
-static	char	sccsid[] = "@(#)chage.c	3.12	08:07:05	19 Jul 1993";
+static	char	sccsid[] = "@(#)chage.c	3.13	00:25:24	05 Feb 1994";
 #endif
 
 /*
@@ -65,7 +66,7 @@ static	char	sccsid[] = "@(#)chage.c	3.12	08:07:05	19 Jul 1993";
 #endif	/* !LOG_WARN */
 #endif	/* USE_SYSLOG */
 
-#ifdef	AGING
+#ifdef	AGING	/*{*/
 
 /*
  * Global variables
@@ -984,7 +985,7 @@ int	state;
 	}
 }
 
-#else	/* !AGING */
+#else	/*} !AGING {*/
 
 /*
  * chage - but there is no age info!
@@ -994,6 +995,8 @@ main (argc, argv)
 int	argc;
 char	**argv;
 {
+	char	*Prog;
+
 	if (Prog = strrchr (argv[0], '/'))
 		Prog++;
 	else
@@ -1003,4 +1006,4 @@ char	**argv;
 	exit (1);
 }
 
-#endif	/* AGING */
+#endif	/*} AGING */
