@@ -1,5 +1,5 @@
 /*
- * Copyright 1989, 1990, 1992, John F. Haugh II
+ * Copyright 1989 - 1994, John F. Haugh II
  * All rights reserved.
  *
  * Permission is granted to copy and create derivative works for any
@@ -7,19 +7,23 @@
  * in all copies of source code, or included in human readable form
  * and conspicuously displayed on all copies of object code or
  * distribution media.
+ *
+ * This software is provided on an AS-IS basis and the author makes
+ * no warrantee of any kind.  The "Artistic License" located in the
+ * file LICENSE contains complete licensing information.
  */
 
 /*
  * faillog.h - login failure logging file format
  *
- *	@(#)faillog.h	3.1	20:36:28	07 Mar 1992
+ *	@(#)faillog.h	3.3	12:59:00	26 Feb 1994
  *
  * The login failure file is maintained by login(1) and faillog(8)
  * Each record in the file represents a separate UID and the file
  * is indexed in that fashion.
  */
 
-#ifdef	SVR4
+#if defined(SVR4) || defined(__linux__)
 #define	FAILFILE	"/var/adm/faillog"
 #else
 #define	FAILFILE	"/usr/adm/faillog"
