@@ -1,5 +1,5 @@
 /*
- * Copyright 1990, 1991, 1992, 1993, John F. Haugh II
+ * Copyright 1990 - 1994, John F. Haugh II
  * All rights reserved.
  *
  * Permission is granted to copy and create derivative works for any
@@ -9,18 +9,18 @@
  * distribution media.
  *
  * This software is provided on an AS-IS basis and the author makes
- * no warrantee of any kind.
+ * no warrantee of any kind.  The "Artistic License" located in the
+ * file LICENSE contains complete licensing information.
  */
 
 #include <sys/types.h>
 #include <stdio.h>
 #include "pwd.h"
-#include "shadow.h"
 #include <grp.h>
 #include <fcntl.h>
 #include <signal.h>
 #include <errno.h>
-#if defined(USG) || defined(SUN4)
+#if defined(USG) || defined(SUN4) || defined(__linux__)
 #include <termio.h>
 #ifdef SYS3
 # include <sys/ioctl.h>
@@ -37,6 +37,10 @@
 #endif /* !SUN && !BSD */
 #include "config.h"
 
+#ifdef	SHADOWGRP
+#include "gshadow.h"
+#endif
+
 #ifdef	USE_SYSLOG
 #include <syslog.h>
 
@@ -50,7 +54,7 @@
 #endif
 
 #ifndef	lint
-static	char	_sccsid[] = "@(#)gpmain.c	3.14	08:16:38	07 May 1993";
+static	char	_sccsid[] = "@(#)gpmain.c	3.16	12:59:02	26 Feb 1994";
 #endif
 
 char	name[BUFSIZ];
@@ -86,7 +90,9 @@ extern	time_t	time ();
 extern	char	*malloc ();
 extern	char	*getpass ();
 #ifdef	NDBM
+#ifdef	SHADOWGRP
 extern	int	sg_dbm_mode;
+#endif
 extern	int	gr_dbm_mode;
 #endif
 
@@ -329,8 +335,10 @@ char	**argv;
 	struct	sgrp	*getsgnam ();
 #endif
 	struct	passwd	*pw = 0;
+#ifndef	__STDC__
 	struct	passwd	*getpwuid ();
 	struct	passwd	*getpwnam ();
+#endif
 
 	/*
 	 * Make a note of whether or not this command was invoked
@@ -342,7 +350,9 @@ char	**argv;
 
 	amroot = getuid () == 0;
 #ifdef	NDBM
+#ifdef	SHADOWGRP
 	sg_dbm_mode = O_RDWR;
+#endif
 	gr_dbm_mode = O_RDWR;
 #endif
 	setuid (geteuid ());
