@@ -1,5 +1,5 @@
 /*
- * Copyright 1989, 1990, 1992, 1993, John F. Haugh II
+ * Copyright 1989 - 1994, John F. Haugh II
  * All rights reserved.
  *
  * Permission is granted to copy and create derivative works for any
@@ -9,13 +9,14 @@
  * distribution media.
  *
  * This software is provided on an AS-IS basis and the author makes
- * not warrantee of any kind.
+ * no warrantee of any kind.  The "Artistic License" located in the
+ * file LICENSE contains complete licensing information.
  */
 
 /*
  * lastlog.h - structure of lastlog file
  *
- *	@(#)lastlog.h	3.1.1.2	08:57:40	10 Jun 1993
+ *	@(#)lastlog.h	3.1.1.4	12:59:06	26 Feb 1994
  *
  *	This file defines a lastlog file structure which should be sufficient
  *	to hold the information required by login.  It should only be used if
@@ -25,7 +26,15 @@
 struct	lastlog	{
 	time_t	ll_time;
 	char	ll_line[8];
-#ifdef	SVR4
+#if defined(SVR4) || defined(__linux__) || defined(SUN4)
 	char	ll_host[16];
 #endif
 };
+
+#ifndef	LASTFILE
+#if defined(SVR4) || defined(__linux__)
+#define LASTFILE	"/var/adm/lastlog"
+#else
+#define	LASTFILE	"/usr/adm/lastlog"
+#endif
+#endif
