@@ -1,5 +1,5 @@
 /*
- * Copyright 1989, 1990, 1991, 1992, 1993, John F. Haugh II
+ * Copyright 1989 - 1994, John F. Haugh II
  * All rights reserved.
  *
  * Permission is granted to copy and create derivative works for any
@@ -9,9 +9,14 @@
  * distribution media.
  *
  * This software is provided on an AS-IS basis and the author makes
- * no warrantee of any kind.
+ * no warrantee of any kind.  The "Artistic License" located in the
+ * file LICENSE contains complete licensing information.
  */
 
+#ifndef	lint
+static	char	sccsid[] = "%W%	%U%	%G%";
+#endif
+
 #include "config.h"
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -70,10 +75,6 @@
 #ifndef	LOG_WARN
 #define	LOG_WARN	LOG_WARNING
 #endif
-#endif
-
-#ifndef	lint
-static	char	sccsid[] = "@(#)lmain.c	3.24	08:07:08	19 Jul 1993";
 #endif
 
 					/* danger - side effects */
@@ -106,9 +107,14 @@ int	preauth_flag;
 #define	GTTY(fd,termio) tcgetattr (fd, termio)
 #define	TERMIO	struct	termios
 #else
+#ifndef BSD
 #define	STTY(fd,termio) ioctl(fd, TCSETA, termio)
 #define	GTTY(fd,termio) ioctl(fd, TCGETA, termio)
 #define	TERMIO	struct	termio
+#else	/* BSD */
+#define	STTY(fd,termio) stty(fd, termio)
+#define	GTTY(fd,termio) gtty(fd, termio)
+#endif	/* !BSD */
 #endif	/* SVR4 || SUN4 */
 TERMIO	termio;
 
@@ -287,12 +293,17 @@ int	namelen;
 	if (speed_table[i].spd_name != -1)
 		speed_name = speed_table[i].spd_name;
 
+	/*
+	 * Put the terminal in cooked mode with echo turned on.
+	 */
+
 	GTTY (0, &termio);
 #ifndef	BSD
 	termio.c_iflag |= ICRNL|IXON;
 	termio.c_oflag |= OPOST|ONLCR;
 	termio.c_lflag |= ICANON|ECHO|ECHOE;
 	termio.c_cflag = (termio.c_cflag & ~CBAUD) | speed_name;
+#else
 #endif
 	STTY (0, &termio);
 
@@ -359,6 +370,7 @@ char	**envp;
 	char	tty[BUFSIZ];
 	int	reason = PW_LOGIN;
 	int	retries;
+	int	arg;
 	int	failed;
 	int	flag;
 	int	subroot = 0;
@@ -378,6 +390,17 @@ char	**envp;
 
 	name[0] = '\0';
 
+	/*
+	 * Check the flags for proper form.  Every argument starting with
+	 * "-" must be exactly two characters long.  This closes all the
+	 * clever rlogin, telnet, and getty holes.
+	 */
+
+	for (arg = 1;arg < argc;arg++) {
+		if (argv[arg][0] == '-' && strlen (argv[arg]) > 2)
+			usage ();
+	}
+
 	/*
 	 * Get the utmp file entry and get the tty name from it.  The
 	 * current process ID must match the process ID in the utmp
@@ -463,15 +486,42 @@ char	**envp;
 	openlog (Prog, LOG_PID|LOG_CONS|LOG_NOWAIT, LOG_AUTH);
 #endif
 
-#ifndef	BSD
 	GTTY (0, &termio);		/* get terminal characteristics */
 
+
 	/*
 	 * Add your favorite terminal modes here ...
 	 */
 
-	termio.c_lflag |= ISIG;
+#ifndef	BSD
+	termio.c_lflag |= ISIG|ICANON|ECHO|ECHOE;
+	termio.c_iflag |= ICRNL;
 
+#if defined(ECHOKE) && defined(ECHOCTL)
+	termio.c_lflag |= ECHOKE|ECHOCTL;
+#endif
+#if defined(ECHOPRT) && defined(NOFLSH) && defined(TOSTOP)
+	termio.c_lflag &= ~(ECHOPRT|NOFLSH|TOSTOP);
+#endif
+#ifdef	ONLCR
+	termio.c_oflag |= ONLCR;
+#endif
+
+#ifdef	SUN4
+
+	/*
+	 * Terminal setup for SunOS 4.1 courtesy of Steve Allen
+	 * at UCO/Lick.
+	 */
+
+	termio.c_cc[VEOF] = '\04';
+	termio.c_cflag &= ~CSIZE;
+	termio.c_cflag |= (PARENB|CS7);
+	termio.c_lflag |= (ISIG|ICANON|ECHO|IEXTEN);
+	termio.c_iflag |= (BRKINT|IGNPAG|ISTRIP|IMAXBEL|ICRNL|IXON);
+	termio.c_iflag &= ~IXANY;
+	termio.c_oflag |= (XTABS|OPOST|ONLCR);
+#endif
 	termio.c_cc[VERASE] = getdef_num("ERASECHAR", '\b');
 	termio.c_cc[VKILL] = getdef_num("KILLCHAR", '\025');
 
@@ -480,8 +530,10 @@ char	**envp;
 	 * effect after the first username login 
 	 */
 
-	STTY (0, &termio);
+#else
 #endif	/* !BSD */
+	STTY (0, &termio);
+
 	umask (getdef_num("UMASK", 0));
 #ifdef HAVE_ULIMIT
 	{
@@ -854,7 +906,7 @@ again:
 	else
 #endif
 	if (pwent.pw_age && pwent.pw_age[0]) {
-		if (expire (&pwent, (void *) 0)) {
+		if (expire (&pwent)) {
 			pwd = getpwnam (name);
 			pwent = *pwd;
 		}
@@ -878,7 +930,7 @@ again:
 #ifdef	SHADOWPWD
 		agecheck (&pwent, spwd);
 #else
-		agecheck (&pwent, (void *) 0);
+		agecheck (&pwent);
 #endif
 #endif	/* AGING */
 		mailcheck ();	/* report on the status of mail */
