@@ -1,5 +1,5 @@
 /*
- * Copyright 1989, 1990, 1991, 1992, 1993, John F. Haugh II
+ * Copyright 1989 - 1994, John F. Haugh II
  * All rights reserved.
  *
  * Permission is granted to copy and create derivative works for any
@@ -9,7 +9,8 @@
  * distribution media.
  *
  * This software is provided on an AS-IS basis and the author makes
- * no warrantee of any kind.
+ * no warrantee of any kind.  The "Artistic License" located in the
+ * file LICENSE contains complete licensing information.
  */
 
 #include "config.h"
@@ -18,9 +19,14 @@
 #include <stdio.h>
 #include <fcntl.h>
 #include <signal.h>
+#ifdef  AIX
+#include <userpw.h>
+#include <usersec.h>
+#include <userconf.h>
+#endif
 
 #ifndef	lint
-static	char	sccsid[] = "@(#)passwd.c	3.11	08:57:44	10 Jun 1993";
+static	char	sccsid[] = "%W%	%U%	%G%";
 #endif
 
 /*
@@ -54,12 +60,14 @@ static	char	sccsid[] = "@(#)passwd.c	3.11	08:57:44	10 Jun 1993";
 
 #include "pwd.h"
 #include "lastlog.h"
+#ifndef	AIX
 #ifdef	SHADOWPWD
 #include "shadow.h"
 #ifndef	AGING
 #define	AGING	0
-#endif
-#endif
+#endif	/* !AGING */
+#endif	/* SHADOWPWD */
+#endif	/* !AIX */
 #include "pwauth.h"
 
 #ifdef	USE_SYSLOG
@@ -70,12 +78,19 @@ static	char	sccsid[] = "@(#)passwd.c	3.11	08:57:44	10 Jun 1993";
 #endif
 #endif
 #ifdef	HAVE_RLIMIT
+#include <limits.h>
 #include <sys/resource.h>
 
 struct	rlimit	rlimit_fsize = { RLIM_INFINITY, RLIM_INFINITY };
 #endif
 
-#ifdef	AGING
+#ifdef  AIX
+int     minage = 0;             /* Minimum age in weeks               */
+int     maxage = 10000;         /* Maximum age in weeks               */
+int     curage;                 /* Current age in weeks               */
+#endif
+
+#if defined(AGING)||defined(AIX)
 
 /*
  * Password aging constants
@@ -95,7 +110,7 @@ struct	rlimit	rlimit_fsize = { RLIM_INFINITY, RLIM_INFINITY };
 #else
 #define	SCALE	DAY
 #endif
-#endif
+#endif	/* AGING||AIX */
 
 /*
  * Global variables
@@ -117,80 +132,83 @@ extern	char	*getlogin();
 extern	char	l64a();
 extern	int	optind;		/* Index into argv[] for current option */
 extern	char	*optarg;	/* Pointer to current option value */
+extern	struct	passwd	*__pw_dup();
+#ifndef	AIX
 #ifdef	NDBM
 extern	int	sp_dbm_mode;
 extern	int	pw_dbm_mode;
 #endif
+#endif
 
 /*
  * #defines for messages.  This facilities foreign language conversion
  * since all messages are defined right here.
  */
 
-#define	USAGE		"usage: %s [ -f | -s ] [ name ]\n"
-#define	ADMUSAGE \
-	"       %s [ -x max ] [ -n min ] [ -w warn ] [ -i inact ] name\n"
-#define	ADMUSAGE2 \
-	"       %s { -l | -d | -S } name\n"
-#define	OLDPASS		"Old Password:"
-#define	NEWPASSMSG \
+char	*USAGE = "usage: %s [ -f | -s ] [ name ]\n";
+char	*ADMUSAGE =
+	"       %s [ -x max ] [ -n min ] [ -w warn ] [ -i inact ] name\n";
+char	*ADMUSAGE2 =
+	"       %s { -l | -d | -S } name\n";
+char	*OLDPASS = "Old Password:";
+char	*NEWPASSMSG =
 "Enter the new password (minimum of %d characters)\n\
-Please use a combination of upper and lower case letters and numbers.\n"
-#define	CHANGING	"Changing password for %s\n"
-#define NEWPASS		"New Password:"
-#define	NEWPASS2	"Re-enter new password:"
-#define	WRONGPWD	"Incorrect password for %s.\n"
-#define	WRONGPWD2	"incorrect password for `%s'\n"
-#define	NOMATCH		"They don't match; try again.\n"
-#define	CANTCHANGE	"The password for %s cannot be changed.\n"
-#define	CANTCHANGE2	"password locked for `%s'\n"
+Please use a combination of upper and lower case letters and numbers.\n";
+char	*CHANGING = "Changing password for %s\n";
+char	*NEWPASS = "New Password:";
+char	*NEWPASS2 = "Re-enter new password:";
+char	*WRONGPWD = "Incorrect password for %s.\n";
+char	*WRONGPWD2 = "incorrect password for `%s'\n";
+char	*NOMATCH = "They don't match; try again.\n";
+char	*CANTCHANGE = "The password for %s cannot be changed.\n";
+char	*CANTCHANGE2 = "password locked for `%s'\n";
 
 #ifdef	AGING
-#define	TOOSOON		"Sorry, the password for %s cannot be changed yet.\n"
-#define	TOOSOON2	"now < sp_min for `%s'\n"
+char	*TOOSOON = "Sorry, the password for %s cannot be changed yet.\n";
+char	*TOOSOON2 = "now < minimum age for `%s'\n";
 #endif
 
-#define	EXECFAILED	"%s: Cannot execute %s"
-#define	EXECFAILED2	"cannot execute %s\n"
-#define	WHOAREYOU	"%s: Cannot determine you user name.\n"
-#define	UNKUSER		"%s: Unknown user %s\n"
-#define	NOPERM		"You may not change the password for %s.\n"
-#define	NOPERM2		"can't change pwd for `%s'\n"
-#define	UNCHANGED	"The password for %s is unchanged.\n"
+char	*EXECFAILED = "%s: Cannot execute %s";
+char	*EXECFAILED2 = "cannot execute %s\n";
+char	*WHOAREYOU = "%s: Cannot determine you user name.\n";
+char	*UNKUSER = "%s: Unknown user %s\n";
+char	*NOPERM = "You may not change the password for %s.\n";
+char	*NOPERM2 = "can't change pwd for `%s'\n";
+char	*UNCHANGED = "The password for %s is unchanged.\n";
 
 #ifdef	SHADOWPWD
-#define	SPWDBUSY	"Cannot lock the password file; try again later.\n"
-#define	SPWDBUSY2	"can't lock /etc/shadow\n"
-#define	OPNERROR	"Cannot open the password file.\n"
-#define	OPNERROR2	"can't open /etc/shadow\n"
-#define	UPDERROR	"Error updating the password entry.\n"
-#define	UPDERROR2	"error updating shadow entry\n"
-#define	DBMERROR	"Error updating the DBM password entry.\n"
-#define	DBMERROR2	"error updating DBM shadow entry.\n"
-#define	CLSERROR	"Cannot commit shadow file changes.\n"
-#define	CLSERROR2	"can't rewrite /etc/shadow.\n"
-#define	UNLKERROR	"Cannot unlock the shadow file.\n"
-#define	UNLKERROR2	"can't unlock /etc/shadow.\n"
+char	*SPWDBUSY = "Cannot lock the password file; try again later.\n";
+char	*SPWDBUSY2 = "can't lock /etc/shadow\n";
+char	*OPNERROR = "Cannot open the password file.\n";
+char	*OPNERROR2 = "can't open /etc/shadow\n";
+char	*UPDERROR = "Error updating the password entry.\n";
+char	*UPDERROR2 = "error updating shadow entry\n";
+char	*DBMERROR = "Error updating the DBM password entry.\n";
+char	*DBMERROR2 = "error updating DBM shadow entry.\n";
+char	*CLSERROR = "Cannot commit shadow file changes.\n";
+char	*CLSERROR2 = "can't rewrite /etc/shadow.\n";
+char	*UNLKERROR = "Cannot unlock the shadow file.\n";
+char	*UNLKERROR2 = "can't unlock /etc/shadow.\n";
 #else
-#define	PWDBUSY		"Cannot lock the password file; try again later.\n"
-#define	PWDBUSY2	"can't lock /etc/passwd\n"
-#define	OPNERROR	"Cannot open the password file.\n"
-#define	OPNERROR2	"can't open /etc/passwd\n"
-#define	UPDERROR	"Error updating the password entry.\n"
-#define	UPDERROR2	"error updating password entry\n"
-#define	DBMERROR	"Error updating the DBM password entry.\n"
-#define	DBMERROR2	"error updating DBM password entry.\n"
-#define	CLSERROR	"Cannot commit password file changes.\n"
-#define	CLSERROR2	"can't rewrite /etc/passwd.\n"
-#define	UNLKERROR	"Cannot unlock the password file.\n"
-#define	UNLKERROR2	"can't unlock /etc/passwd.\n"
+char	*PWDBUSY = "Cannot lock the password file; try again later.\n";
+char	*PWDBUSY2 = "can't lock /etc/passwd\n";
+char	*OPNERROR = "Cannot open the password file.\n";
+char	*OPNERROR2 = "can't open /etc/passwd\n";
+char	*UPDERROR = "Error updating the password entry.\n";
+char	*UPDERROR2 = "error updating password entry\n";
+char	*DBMERROR = "Error updating the DBM password entry.\n";
+char	*DBMERROR2 = "error updating DBM password entry.\n";
+char	*CLSERROR = "Cannot commit password file changes.\n";
+char	*CLSERROR2 = "can't rewrite /etc/passwd.\n";
+char	*UNLKERROR = "Cannot unlock the password file.\n";
+char	*UNLKERROR2 = "can't unlock /etc/passwd.\n";
 #endif
 
-#define	NOTROOT		"Cannot change ID to root.\n"
-#define	NOTROOT2	"can't setuid(0).\n"
-#define	TRYAGAIN	"Try again.\n"
-#define	CHGPASSWD	"changed password for `%s'\n"
-#define	NOCHGPASSWD	"did not change password for `%s'\n"
+char	*NOTROOT = "Cannot change ID to root.\n";
+char	*NOTROOT2 = "can't setuid(0).\n";
+char	*TRYAGAIN = "Try again.\n";
+char	*CHGPASSWD = "changed password for `%s'\n";
+char	*NOCHGPASSWD = "did not change password for `%s'\n";
 
 /*
  * usage - print command usage and exit
@@ -263,13 +281,14 @@ struct	spwd	*sp;
 {
 	sp->sp_lstchg = time ((time_t *) 0) / SCALE;
 }
-#else
+#else	/* !SHADOWPWD */
 update_age (pw)
 struct	passwd	*pw;
 {
 #ifdef	ATT_AGE
 	long	week;		/* at the office ... */
 	static	char	age[5];	/* Password age string */
+	char	*cp;
 
 	week = time ((time_t *) 0) / WEEK;
 	if (pw->pw_age[0]) {
@@ -286,6 +305,7 @@ struct	passwd	*pw;
 #endif	/* SHADOWPWD */
 #endif	/* AGING */
 
+#ifndef	AIX
 /*
  * insert_crypt_passwd - add an "old-style" password to authentication string
  */
@@ -295,22 +315,28 @@ char	*string;
 char	*passwd;
 char	*result;
 {
-	while (*string) {
-		if (string[0] == ';') {
-			*result++ = *string++;
-		} else if (string[0] == '@') {
-			while (*string && *string != ';')
+	if (! string || ! *string) {
+		while (*passwd)
+			*result++ = *passwd++;
+	} else {
+		while (*string) {
+			if (string[0] == ';') {
 				*result++ = *string++;
-		} else {
-			while (*passwd)
-				*result++ = *passwd++;
+			} else if (string[0] == '@') {
+				while (*string && *string != ';')
+					*result++ = *string++;
+			} else {
+				while (*passwd)
+					*result++ = *passwd++;
 
-			while (*string && *string != ';')
-				string++;
+				while (*string && *string != ';')
+					string++;
+			}
 		}
 	}
 	*result = '\0';
 }
+#endif	/* !AIX */
 
 /*
  * new_password - validate old password and replace with new
@@ -325,7 +351,7 @@ struct	spwd	*sp;
 #else
 new_password (pw)
 struct	passwd	*pw;
-#endif
+#endif	/* SHADOWPWD */
 {
 	char	*clear;		/* Pointer to clear text */
 	char	*cipher;	/* Pointer to cipher text */
@@ -343,8 +369,8 @@ struct	passwd	*pw;
 	 * own password.
 	 */
 
+	bzero (orig, sizeof orig);
 	if (! amroot && crypt_passwd[0]) {
-		bzero (orig, sizeof orig);
 
 		if (! (clear = getpass (OLDPASS)))
 			return -1;
@@ -413,12 +439,16 @@ struct	passwd	*pw;
 	update_age (sp);
 #else
 	cp = pw_encrypt (pass, (char *) 0);
+#ifndef	AIX
 	insert_crypt_passwd (pw->pw_passwd, cp, new_auth);
 	cp = pw->pw_passwd;
 	pw->pw_passwd = strdup (new_auth);
+#else
+	pw->pw_passwd = strdup (cp);
+#endif	/* !AIX */
 #ifdef	AGING
 	update_age (pw);
-#endif
+#endif	/* AGING */
 #endif	/* SHADOWPWD */
 	bzero (pass, sizeof pass);
 	bzero (new_auth, sizeof new_auth);
@@ -427,7 +457,7 @@ struct	passwd	*pw;
 	return 0;
 }
 
-#ifdef	AGING
+#if defined(AGING)||defined(AIX)
 
 /*
  * check_password - test a password to see if it can be changed
@@ -443,9 +473,15 @@ check_password (pw, sp)
 struct	passwd	*pw;
 struct	spwd	*sp;
 #else
+#ifdef	AIX
+check_password (pw, sp)
+struct	passwd	*pw;
+struct	userpw	*sp;
+#else
 check_password (pw)
 struct	passwd	*pw;
-#endif
+#endif	/* AIX */
+#endif	/* SHADOWPWD */
 {
 	time_t	now = time ((time_t *) 0) / SCALE;
 #ifndef	SHADOWPWD
@@ -469,7 +505,7 @@ struct	passwd	*pw;
 
 #ifdef	SHADOWPWD
 	if ((sp->sp_expire > 0 && now >= sp->sp_expire) ||
-	    (sp->sp_inact >= 0 && sp->sp_max >= 0 &&
+	    (sp->sp_inact >= 0 && sp->sp_max >= 0 && sp->sp_lstchg > 0 &&
 		now >= (sp->sp_lstchg + sp->sp_inact + sp->sp_max)) ||
 			strcmp (sp->sp_pwdp, "!") == 0 ||
 			sp->sp_min > sp->sp_max) {
@@ -496,7 +532,7 @@ struct	passwd	*pw;
 		exit (1);
 	}
 #else	/* !SHADOWPWD */
-
+#ifdef	ATT_AGE
 	/*
 	 * Can always be changed if there is no age info
 	 */
@@ -506,7 +542,15 @@ struct	passwd	*pw;
 
 	last = a64l (pw->pw_age + 2) * WEEK;
 	ok = last + c64i (pw->pw_age[1]) * WEEK;
-
+#else	/* !ATT_AGE */
+#ifdef	AIX
+        last = sp->upw_lastupdate / SCALE;
+	ok = (last + (minage > 0 ? minage * (7*86400L):0) / SCALE);
+#else
+	last = 0;
+	ok = 0;
+#endif	/* AIX */
+#endif	/* ATT_AGE */
 	if (now < ok) {
 		fprintf (stderr, TOOSOON, pw->pw_name);
 #ifdef	USE_SYSLOG
@@ -606,11 +650,17 @@ print_status (pw, sp)
 struct	passwd	*pw;
 struct	spwd	*sp;
 #else
+#ifdef	AIX
+print_status (pw, sp)
+struct	passwd	*pw;
+struct	userpw	*sp;
+#else
 print_status (pw)
 struct	passwd	*pw;
-#endif
+#endif	/* AIX */
+#endif	/* SHADOWPWD */
 {
-#ifdef	AGING
+#if defined(AGING)||defined(AIX)
 	struct	tm	*tm;
 	time_t	last_time;
 #endif
@@ -626,10 +676,21 @@ struct	passwd	*pw;
 	printf ("%d %d %d %d\n",
 		(sp->sp_min * SCALE) / DAY, (sp->sp_max * SCALE) / DAY,
 		(sp->sp_warn * SCALE) / DAY, (sp->sp_inact * SCALE) / DAY);
-#else
+#else	/* ! SHADOWPWD */
 	printf ("%s ", pw->pw_name);
 	printf ("%s",
-		pw->pw_passwd[0] ? (pw->pw_passwd[0] == '!' ? "L":"P"):"NP");
+#ifdef	AIX
+		pw->pw_passwd[0] ? (pw->pw_passwd[0] == '*' ? "L":"P"):"NP"
+#else
+		pw->pw_passwd[0] ? (pw->pw_passwd[0] == '!' ? "L":"P"):"NP"
+#endif
+	);
+
+#ifdef	AGING	/*{*/
+
+	/*
+	 * Output the password change and expiry information.
+	 */
 
 #ifdef	ATT_AGE
 	if (pw->pw_age[0])
@@ -644,8 +705,22 @@ struct	passwd	*pw;
 		pw->pw_age[0] ? c64i (pw->pw_age[1]) * 7:10000,
 		pw->pw_age[0] ? c64i (pw->pw_age[0]) * 7:0);
 #else	/* !ATT_AGE */
+#ifdef	AIX
+	last_time = sp->upw_lastupdate;
+	tm = gmtime (&last_time);
+
+	printf (" %02.2d/%02.2d/%02.2d ",
+		tm->tm_mon + 1, tm->tm_mday, tm->tm_year % 100);
+	printf ("%d %d\n",
+		maxage > 0 ? maxage:10000,
+		minage > 0 ? minage:0);
+#else
 	putchar ('\n');
+#endif	AIX
 #endif	/* ATT_AGE */
+#else	/* ! AGING */
+	putchar ('\n');
+#endif	/*} AGING */
 #endif	/* SHADOWPWD */
 }
 
@@ -706,6 +781,9 @@ char	**argv;
 #else
 	char	age[5];			/* New password age entry             */
 #endif
+#ifdef  AIX
+	struct  userpw  userpw, *pu;
+#endif
 
 	/*
 	 * The program behaves differently when executed by root
@@ -784,6 +862,7 @@ char	**argv;
 #endif
 	{
 		switch (flag) {
+#ifdef	AGING
 			case 'x':
 				max = strtol (optarg, &cp, 10);
 #ifndef	SHADOWPWD
@@ -826,6 +905,7 @@ char	**argv;
 
 				break;
 #endif	/* SHADOWPWD */
+#endif	/* AGING */
 			case 'S':
 				if (getuid ())
 					usage ();
@@ -866,15 +946,23 @@ char	**argv;
 	if ((dflg || lflg || xflg || nflg ||
 				wflg || iflg || Sflg) && optind >= argc)
 #else
+#ifdef  AGING
 	if ((dflg || lflg || xflg || nflg || Sflg) && optind >= argc)
-#endif
+#else
+	if ((dflg || lflg || Sflg) && optind >= argc)
+#endif  /* AGING */
+#endif	/* SHADOWPWD */
 		usage ();
 
 #ifdef	SHADOWPWD
 	if ((dflg + lflg + uflg + (xflg || nflg || wflg || iflg) + Sflg) > 1)
 #else
-	if ((dflg + lflg + uflg + (xflg || nflg) + Sflg) > 1)
-#endif
+#ifdef  AGING
+        if ((dflg + lflg + uflg + (xflg || nflg) + Sflg) > 1)
+#else
+        if ((dflg + lflg + uflg + Sflg) > 1)
+#endif  /* AGING */
+#endif	/* SHADOWPWD */
 		usage ();
 
 	/*
@@ -911,6 +999,7 @@ char	**argv;
 #endif
 		exit (1);
 	}
+	pw = __pw_dup (pw);
 	if (! amroot && pw->pw_uid != getuid ()) {
 		fprintf (stderr, NOPERM, name);
 #ifdef	USE_SYSLOG
@@ -920,6 +1009,31 @@ char	**argv;
 		exit (1);
 	}
 
+#ifdef  AIX
+
+        /*
+         * The aging information lives someplace else.  Get it from the
+         * login.cfg file
+         */
+
+        if (getconfattr (SC_SYS_PASSWD, SC_MINAGE, &minage, SEC_INT))
+                minage = -1;
+
+        if (getconfattr (SC_SYS_PASSWD, SC_MAXAGE, &maxage, SEC_INT))
+                maxage = -1;
+
+        pu = getuserpw (name);
+        curage = (time (0) - pu->upw_lastupdate) / (7*86400L);
+        if (! amroot && minage > 0 && curage < minage) {
+                fprintf (stderr, CANTCHANGE, pw->pw_name);
+#ifdef  USE_SYSLOG
+                syslog (LOG_WARN, CANTCHANGE2, pw->pw_name);
+                closelog ();
+#endif
+                exit (1);
+        }
+#endif	/* AIX */
+
 	/*
 	 * Let the user know whose password is being changed.
 	 */
@@ -952,9 +1066,14 @@ char	**argv;
 	if (Sflg) {
 #ifdef	SHADOWPWD
 		print_status (pw, sp);
+#else
+#ifdef	AIX
+		pu = getuserpw (name);
+		print_status (pw, pu);
 #else
 		print_status (pw);
-#endif
+#endif	/* AIX */
+#endif	/* SHADOWPWD */
 #ifdef	USE_SYSLOG
 		closelog ();
 #endif
@@ -968,12 +1087,16 @@ char	**argv;
 #ifdef	SHADOWPWD
 	if (! (dflg || lflg || uflg || xflg || nflg || wflg || iflg))
 #else
-	if (! (dflg || lflg || uflg || xflg || nflg))
-#endif
+#ifdef  AGING
+        if (! (dflg || lflg || uflg || xflg || nflg))
+#else
+        if (! (dflg || lflg || uflg))
+#endif  /* AGING */
+#endif  /* SHADOWPWD */
 	{
 #ifdef	SHADOWPWD
 		if (strchr (sp->sp_pwdp, '@')) {
-			if (pw_auth (sp->sp_pwdp, name, PW_CHANGE, 0)) {
+			if (pw_auth (sp->sp_pwdp, name, PW_CHANGE, (char *) 0)) {
 #ifdef	USE_SYSLOG
 				syslog (LOG_INFO, NOCHGPASSWD, name);
 				closelog ();
@@ -989,20 +1112,21 @@ char	**argv;
 			strcpy (crypt_passwd, sp->sp_pwdp);
 #else	/* !SHADOWPWD */
 		if (strchr (pw->pw_passwd, '@')) {
-			if (pw_auth (pw->pw_passwd, name, PW_CHANGE, 0)) {
+			if (pw_auth (pw->pw_passwd, name, PW_CHANGE, (char *) 0)) {
 #ifdef	USE_SYSLOG
 				syslog (LOG_INFO, CHGPASSWD, name);
 				closelog ();
 #endif
 				exit (0);
 			} else {
+#ifdef	AGING
 				update_age (pw);
-
+#endif
 				if (! uses_default_method (pw->pw_passwd))
 					goto done;
 			}
 		} else
-			strcpy (crypt_passwd, pw_pw_passwd);
+			strcpy (crypt_passwd, pw->pw_passwd);
 #endif	/* SHADOWPWD */
 
 		/*
@@ -1052,7 +1176,7 @@ char	**argv;
 			sp->sp_pwdp = new_passwd;
 		}
 #else
-		if (pw->pw_passwd & pw->pw_passwd[0] != '!') {
+		if (pw->pw_passwd && pw->pw_passwd[0] != '!') {
 			strcpy (new_passwd, "!");
 			strcat (new_passwd, pw->pw_passwd);
 			pw->pw_passwd = new_passwd;
@@ -1076,6 +1200,7 @@ char	**argv;
 	bzero (age, sizeof age);
 	strcpy (age, pw->pw_age);
 #endif
+#ifdef	AGING
 	if (xflg) {
 #ifdef	SHADOWPWD
 		sp->sp_max = (max * DAY) / SCALE;
@@ -1093,6 +1218,7 @@ char	**argv;
 		age[1] = i64c (min);
 #endif
 	}
+#endif	/* AGING */
 #ifdef	SHADOWPWD
 	if (wflg)
 		sp->sp_warn = (warn * DAY) / SCALE;
@@ -1133,6 +1259,30 @@ done:
 	signal (SIGTSTP, SIG_IGN);
 #endif
 
+#ifdef  AIX
+
+        /*
+         * AIX very conveniently has its own mechanism for updating
+         * passwords.  Use it instead ...
+         */
+
+        strcpy (userpw.upw_name, pw->pw_name);
+        userpw.upw_passwd = pw->pw_passwd;
+        userpw.upw_lastupdate = time (0);
+        userpw.upw_flags = 0;
+
+	setpwdb (S_WRITE);
+
+        if (putuserpw (&userpw)) {
+                fprintf (stderr, UPDERROR);
+#ifdef  USE_SYSLOG
+                syslog (LOG_ERR, UPDERROR2);
+                closelog ();
+#endif
+                exit (1);
+        }
+	endpwdb ();
+#else   /* !AIX */
 	/*
 	 * The shadow entry is now ready to be committed back to
 	 * the shadow file.  Get a lock on the file and open it.
@@ -1265,6 +1415,7 @@ done:
 #endif
 		exit (1);
 	}
+#endif	/* AIX */
 #ifdef	USE_SYSLOG
 	syslog (LOG_INFO, CHGPASSWD, name);
 	closelog ();
