@@ -8,6 +8,10 @@
  * and conspicuously displayed on all copies of object code or
  * distribution media.
  *
+ * This software is provided on an AS-IS basis and the author makes
+ * no warrantee of any kind.  The "Artistic License" located in the
+ * file LICENSE contains complete licensing information.
+ *
  * Revision History
  *	11/25/91	3.1.1	patchlevel 14
  *		Added "login.defs" to Makefile
@@ -27,8 +31,10 @@
  *		New baseline release
  *	08/23/93	3.3.1	patchlevel 24
  *		Bug fixes for SunOS 4.1.1
+ *	08/27/93	3.3.2	patchlevel 25
+ *		Initial NIS support changes
  */
 
 #define	RELEASE		3
-#define	PATCHLEVEL	24
-#define	VERSION		"3.3.1"
+#define	PATCHLEVEL	25
+#define	VERSION		"3.3.2"
