@@ -1,23 +1,31 @@
 /*
- * Copyright 1990, 1991, John F. Haugh II
+ * Copyright 1990 - 1994, John F. Haugh II
  * All rights reserved.
  *
- * Use, duplication, and disclosure prohibited without
- * the express written permission of the author.
+ * Permission is granted to copy and create derivative works for any
+ * non-commercial purpose, provided this copyright notice is preserved
+ * in all copies of source code, or included in human readable form
+ * and conspicuously displayed on all copies of object code or
+ * distribution media.
+ *
+ * This software is provided on an AS-IS basis and the author makes
+ * no warrantee of any kind.  The "Artistic License" located in the
+ * file LICENSE contains complete licensing information.
  */
 
-#ifndef	lint
-static	char	sccsid[] = "@(#)spdbm.c	3.3	08:46:22	12 Sep 1991";
-#endif
-
 #include <string.h>
 #include <stdio.h>
 #include "config.h"
+
+#if defined(SHADOWPWD) && defined(NDBM)	/*{*/
+#ifndef	lint
+static	char	sccsid[] = "@(#)spdbm.c	3.4	20:08:34	08 Feb 1994";
+#endif
+
 #include "shadow.h"
 
-#ifdef	NDBM
 #include <ndbm.h>
-DBM	*sp_dbm;
+extern	DBM	*sp_dbm;
 
 /*
  * sp_dbm_update
@@ -86,4 +94,4 @@ char	*user;
 
 	return 1;
 }
-#endif
+#endif	/*} SHADOWPWD && NDBM */
