@@ -1,5 +1,5 @@
 /*
- * Copyright 1991, 1992, 1993, John F. Haugh II
+ * Copyright 1991 - 1994, John F. Haugh II
  * All rights reserved.
  *
  * Permission is granted to copy and create derivative works for any
@@ -9,11 +9,12 @@
  * distribution media.
  *
  * This software is provided on an AS-IS basis and the author makes
- * no warrantee of any kind.
+ * no warrantee of any kind.  The "Artistic License" located in the
+ * file LICENSE contains complete licensing information.
  */
 
 #ifndef lint
-static	char	sccsid[] = "@(#)useradd.c	3.14	08:20:53	23 Aug 1993";
+static	char	sccsid[] = "%W%	%U%	%G%";
 #endif
 
 #include "config.h"
@@ -36,6 +37,9 @@ static	char	sccsid[] = "@(#)useradd.c	3.14	08:20:53	23 Aug 1993";
 #ifdef	SHADOWPWD
 #include "shadow.h"
 #endif
+#ifdef	SHADOWGRP
+#include "gshadow.h"
+#endif
 #include "pwauth.h"
 
 #ifdef	USE_SYSLOG
@@ -51,9 +55,7 @@ char	def_gname[16];
 char	def_home[BUFSIZ];
 char	def_shell[BUFSIZ];
 char	def_template[BUFSIZ] = "/etc/skel";
-#ifdef	SHADOWPWD
 long	def_inactive;
-#endif
 long	def_expire;
 #ifdef	SVR4
 char	def_file[] = "/usr/sadm/defadduser";
@@ -61,9 +63,19 @@ char	def_file[] = "/usr/sadm/defadduser";
 char	def_file[] = "/etc/default/useradd";
 #endif
 
+/*
+ * This is a slightly schizophrenic definition.  We want NGROUPS_MAX,
+ * but probably have NGROUPS and may not have NGROUPS_MAX.  The rest
+ * of Shadow keys on NGROUPS.
+ */
+
 #ifndef	NGROUPS_MAX
+#ifdef	NGROUPS
+#define	NGROUPS_MAX	NGROUP
+#else
 #define	NGROUPS_MAX	64
 #endif
+#endif
 
 #if !defined(MDY_DATE) && !defined(DMY_DATE) && !defined(YMD_DATE)
 #define	MDY_DATE	1
@@ -134,8 +146,10 @@ extern	int	fclose();
 extern	char	*malloc();
 extern	char	*mktemp();
 
+#ifndef	__STDC__
 extern	struct	group	*getgrnam();
 extern	struct	group	*getgrgid();
+#endif
 extern	struct	group	*gr_next();
 extern	struct	group	*gr_locate();
 extern	int	gr_lock();
@@ -401,9 +415,7 @@ get_defaults ()
 		def_group = 1;
 		strcpy (def_gname, "other");
 		strcpy (def_home, "/home");
-#ifdef	SHADOWPWD
 		def_inactive = 0;
-#endif
 		def_expire = 0;
 		return;
 	}
@@ -505,6 +517,22 @@ get_defaults ()
 			else
 				def_expire = atoi (cp);
 		}
+
+		/*
+		 * Default Skeleton information
+		 */
+
+#ifdef	SVR4
+		else if (strncmp ("defskel=", buf, 8) == 0)
+#else
+		else if (strncmp ("SKEL=", buf, 5) == 0)
+#endif
+		{
+			if (*cp == '\0')
+				cp = "/etc/skel";
+			
+			strcpy (def_template, cp);
+		}
 	}
 }
 
@@ -528,7 +556,11 @@ show_defaults ()
 	printf ("group=%s,%d  basedir=%s  skel=%s\n",
 		def_gname, def_group, def_home, def_template);
 
+#ifdef	SHADOWPWD
 	printf ("shell=%s  inactive=%d  ", def_shell, def_inactive);
+#else
+	printf ("shell=%s  ", def_shell);
+#endif
 
 	if (def_expire >= 0)
 		printf ("expire=%d/%d/%d\n",
@@ -539,9 +571,7 @@ show_defaults ()
 #else
 	printf ("GROUP=%d\n", def_group);
 	printf ("HOME=%s\n", def_home);
-#ifdef	SHADOWPWD
 	printf ("INACTIVE=%d\n", def_inactive);
-#endif
 	printf ("EXPIRE=%d\n", def_expire);
 	printf ("SHELL=%s\n", def_shell);
 	printf ("SKEL=%s\n", def_template);
@@ -1201,8 +1231,13 @@ find_new_uid ()
 				Prog, user_id);
 			exit (1);
 		}
-		if (! uflg && pwd->pw_uid >= user_id)
+		if (! uflg && pwd->pw_uid >= user_id) {
+#ifdef	SHORT_UIDS
+			if ((unsigned long) pwd->pw_uid > 65000)
+				continue;
+#endif
 			user_id = pwd->pw_uid + 1;
+		}
 	}
 }
 
@@ -1256,7 +1291,7 @@ char	*methods;
 	 * Cursory checks, length and illegal characters
 	 */
 
-	if (strlen (methods) > 256)
+	if ((int) strlen (methods) > 256)
 		return 0;
 
 	if (! VALID (methods))
@@ -1274,8 +1309,7 @@ char	*methods;
 		if (strcmp (cp, "DEFAULT") == 0) {
 			if (default_cnt++ > 0)
 				return 0;
-		} else if (cp[0] != '/')
-			return 0;
+		}
 	}
 	return 1;
 }
@@ -1359,10 +1393,10 @@ char	**argv;
 			case 'e':
 				eflg++;
 				if (Dflg)
-					def_expire = strtoday (optarg);
+					def_expire = atoi (optarg);
 				else {
 #ifdef	SHADOWPWD
-					user_expire = atoi (optarg);
+					user_expire = strtoday (optarg);
 #ifdef	ITI_AGING
 					user_expire *= DAY;
 #endif
@@ -1419,7 +1453,7 @@ char	**argv;
 				break;
 			case 's':
 				if (! VALID (optarg) || (optarg[0] &&
-						optarg[0] != '/')) {
+				    optarg[0] != '/') && optarg != '*') {
 					fprintf (stderr,
 						"%s: invalid field `%s'\n",
 						Prog, optarg);
@@ -1460,6 +1494,9 @@ char	**argv;
 	if (! gflg)
 		user_gid = def_group;
 
+	if (! sflg)
+		sprintf (user_shell, "%s", def_shell);
+
 	if (Dflg) {
 		if (optind != argc)
 			usage ();
@@ -1595,7 +1632,7 @@ usr_update ()
 	 * than the update of the password file, we do this first.
 	 */
 
-	if (Aflg && pw_auth (user_auth, pwent.pw_name, PW_ADD, 0)) {
+	if (Aflg && pw_auth (user_auth, pwent.pw_name, PW_ADD, (char *) 0)) {
 		fprintf (stderr, "%s: error adding authentication method\n",
 			Prog);
 		fail (1);
@@ -1775,7 +1812,7 @@ char	**argv;
 	 */
 
 	if (Dflg) {
-		if (gflg || bflg || fflg || eflg)
+		if (gflg || bflg || fflg || eflg || sflg)
 			exit (set_defaults () ? 1:0);
 
 		show_defaults ();
