@@ -1,5 +1,5 @@
 /*
- * Copyright 1991, 1992, 1993, John F. Haugh II
+ * Copyright 1991 - 1994, John F. Haugh II
  * All rights reserved.
  *
  * Permission is granted to copy and create derivative works for any
@@ -9,11 +9,12 @@
  * distribution media.
  *
  * This software is provided on an AS-IS basis and the author makes
- * no warrantee of any kind.
+ * no warrantee of any kind.  The "Artistic License" located in the
+ * file LICENSE contains complete licensing information.
  */
 
 #ifndef lint
-static	char	sccsid[] = "@(#)usermod.c	3.16	08:11:52	07 May 1993";
+static	char	sccsid[] = "@(#)usermod.c	3.18	12:59:21	26 Feb 1994";
 #endif
 
 #include <sys/types.h>
@@ -36,6 +37,9 @@ static	char	sccsid[] = "@(#)usermod.c	3.16	08:11:52	07 May 1993";
 #ifdef	SHADOWPWD
 #include "shadow.h"
 #endif
+#ifdef	SHADOWGRP
+#include "gshadow.h"
+#endif
 #include "faillog.h"
 #include "lastlog.h"
 #include "pwauth.h"
@@ -48,9 +52,19 @@ static	char	sccsid[] = "@(#)usermod.c	3.16	08:11:52	07 May 1993";
 #endif
 #endif
 
+/*
+ * This is a slightly schizophrenic definition.  We want NGROUPS_MAX,
+ * but probably have NGROUPS and may not have NGROUPS_MAX.  The rest
+ * of Shadow keys on NGROUPS.
+ */
+
 #ifndef	NGROUPS_MAX
+#ifdef	NGROUPS
+#define	NGROUPS_MAX	NGROUP
+#else
 #define	NGROUPS_MAX	64
 #endif
+#endif
 
 #if defined(DIR_XENIX) || defined(DIR_BSD) || defined(DIR_SYSV)
 #define	DIR_ANY
@@ -120,8 +134,10 @@ extern	int	fclose();
 extern	char	*malloc();
 extern	char	*mktemp();
 
+#ifndef	__STDC__
 extern	struct	group	*getgrnam();
 extern	struct	group	*getgrgid();
+#endif
 extern	struct	group	*gr_next();
 extern	struct	group	*gr_locate();
 extern	int	gr_lock();
@@ -175,63 +191,6 @@ static	short	juldays[13] = { 0,
 	0,	31,	59,	90,	120,	151,	/* JAN - JUN */
 	181,	212,	243,	273,	304,	334 };	/* JUL - DEC */
 
-#ifdef	NEED_RENAME
-/*
- * rename - rename a file to another name
- *
- *	rename is provided for systems which do not include the rename()
- *	system call.
- */
-
-int
-rename (begin, end)
-char	*begin;
-char	*end;
-{
-	struct	stat	s1, s2;
-	extern	int	errno;
-	int	orig_err = errno;
-
-	if (stat (begin, &s1))
-		return -1;
-
-	if (stat (end, &s2)) {
-		errno = orig_err;
-	} else {
-
-		/*
-		 * See if this is a cross-device link.  We do this to
-		 * insure that the link below has a chance of working.
-		 */
-
-		if (s1.st_dev != s2.st_dev) {
-			errno = EXDEV;
-			return -1;
-		}
-
-		/*
-		 * See if we can unlink the existing destination
-		 * file.  If the unlink works the directory is writable,
-		 * so there is no need here to figure that out.
-		 */
-
-		if (unlink (end))
-			return -1;
-	}
-
-	/*
-	 * Now just link the original name to the final name.  If there
-	 * was no file previously, this link will fail if the target
-	 * directory isn't writable.  The unlink will fail if the source
-	 * directory isn't writable, but life stinks ...
-	 */
-
-	if (link (begin, end) || unlink (begin))
-		return -1;
-
-	return 0;
-}
-#endif
 
 /*
  * strtoday - compute the number of days since 1970.
@@ -467,15 +426,26 @@ char	*list;
 			fprintf (stderr, "%s: unknown group %s\n", Prog, list);
 			errors++;
 		}
+		list = cp;
 
+#ifdef	USE_NIS
+		/*
+		 * Don't add this group if they are an NIS group.  Tell
+		 * the user to go to the server for this group.
+		 */
+
+		if (__isgrNIS ()) {
+			fprintf (stderr, "%s: group `%s' is a NIS group.\n",
+				Prog, grp->gr_name);
+			continue;
+		}
+#endif
 		/*
 		 * Add the GID value from the group file to the user's
 		 * list of groups.
 		 */
 
 		user_groups[user_ngroups++] = grp->gr_gid;
-
-		list = cp;
 	} while (list);
 
 	/*
@@ -1097,7 +1067,7 @@ char	*methods;
 	 * Cursory checks, length and illegal characters
 	 */
 
-	if (strlen (methods) > 256)
+	if ((int) strlen (methods) > 256)
 		return 0;
 
 	if (! VALID (methods))
@@ -1115,8 +1085,7 @@ char	*methods;
 		if (strcmp (cp, "DEFAULT") == 0) {
 			if (default_cnt++ > 0)
 				return 0;
-		} else if (cp[0] != '/')
-			return 0;
+		}
 	}
 	return 1;
 }
@@ -1152,6 +1121,30 @@ char	**argv;
 		exit (6);
 	}
 	strcpy (user_name, pwd->pw_name);
+
+#ifdef	USE_NIS
+
+	/*
+	 * Now make sure it isn't an NIS user.
+	 */
+
+	if (__ispwNIS ()) {
+		char	*nis_domain;
+		char	*nis_master;
+		int	nis_masterlen;
+
+		fprintf (stderr, "%s: user %s is a NIS user\n",
+			Prog, user_name);
+
+		if (! yp_get_default_domain (&nis_domain) &&
+				! yp_master (nis_domain, "passwd.byname",
+				&nis_master)) {
+			fprintf (stderr, "%s: %s is the NIS master\n",
+				Prog, nis_master);
+		}
+		exit (6);
+	}
+#endif
 	user_id = pwd->pw_uid;
 	user_gid = pwd->pw_gid;
 	strcpy (user_comment, pwd->pw_gecos);
@@ -1343,7 +1336,9 @@ close_files ()
 	}
 	(void) sgr_unlock ();
 #endif
+#ifdef	SHADOWPWD
 	(void) spw_unlock ();
+#endif
 	(void) pw_unlock ();
 
 	/*
@@ -1435,11 +1430,13 @@ usr_update ()
 #else
 	strcpy (old_auth, pwent.pw_passwd);
 #endif
-	if (lflg || (Aflg && strcmp (old_auth, user_auth) != 0)) {
+	if (Aflg)
 		convert_auth (user_auth, old_auth, auth_arg);
+
+	if (lflg || (Aflg && strcmp (old_auth, user_auth) != 0)) {
 		delete_auths (old_auth, user_auth, auth_buf);
 		if (auth_buf[0] && pw_auth (auth_buf, user_name,
-				PW_DELETE, 0)) {
+				PW_DELETE, (char *) 0)) {
 			fprintf (stderr,
 				"%s: error deleting authentication method\n",
 				Prog);
@@ -1451,7 +1448,7 @@ usr_update ()
 		}
 		add_auths (old_auth, user_auth, auth_buf);
 		if (auth_buf[0] == '@' && pw_auth (auth_buf,
-				lflg ? user_newname:user_name, PW_ADD, 0)) {
+			lflg ? user_newname:user_name, PW_ADD, (char *) 0)) {
 			fprintf (stderr,
 				"%s: error adding authentication method\n",
 				Prog);
@@ -1619,7 +1616,7 @@ update_files ()
 	 * anything to just leave it be.
 	 */
 
-	if ((fd = open ("/usr/adm/lastlog", O_RDWR)) != -1) {
+	if ((fd = open (LASTFILE, O_RDWR)) != -1) {
 		lseek (fd, (long) user_id * sizeof ll, 0);
 		if (read (fd, &ll, sizeof ll) == sizeof ll) {
 			lseek (fd, (long) user_newid * sizeof ll, 0);
